
-----------------------------------------------------------------------------------------------
-- Client Lua Script for ListNearbyPlayers
-- Copyright (c) NCsoft. All rights reserved
-----------------------------------------------------------------------------------------------
 
require "Window"
 
-----------------------------------------------------------------------------------------------
-- ListNearbyPlayers Module Definition
-----------------------------------------------------------------------------------------------
local ListNearbyPlayers = {} 
 
local nearbyPlayers = {}
local nearbyPlayersFirstTimeSeen = {}



-----------------------------------------------------------------------------------------------
-- Constants
-----------------------------------------------------------------------------------------------
-- e.g. local kiExampleVariableMax = 999
 
-----------------------------------------------------------------------------------------------
-- Initialization
-----------------------------------------------------------------------------------------------
function ListNearbyPlayers:new(o)
    o = o or {}
    setmetatable(o, self)
    self.__index = self 
	self.reportedPlayers = {}
    -- initialize variables here

    return o
end

function ListNearbyPlayers:Init()
	local bHasConfigureFunction = false
	local strConfigureButtonText = ""
	local tDependencies = {
		-- "UnitOrPackageName",
	}
    Apollo.RegisterAddon(self, bHasConfigureFunction, strConfigureButtonText, tDependencies)
end
 

-----------------------------------------------------------------------------------------------
-- ListNearbyPlayers OnLoad
-----------------------------------------------------------------------------------------------
function ListNearbyPlayers:OnLoad()
    -- load our form file

	self.xmlDoc = XmlDoc.CreateFromFile("ListNearbyPlayers.xml")
	self.xmlDoc:RegisterCallback("OnDocLoaded", self)
end

-----------------------------------------------------------------------------------------------
-- ListNearbyPlayers OnDocLoaded
-----------------------------------------------------------------------------------------------
function ListNearbyPlayers:OnDocLoaded()

	if self.xmlDoc ~= nil and self.xmlDoc:IsLoaded() then
	    self.wndMain = Apollo.LoadForm(self.xmlDoc, "ListNearbyPlayersForm", nil, self)
		if self.wndMain == nil then
			Apollo.AddAddonErrorText(self, "Could not load the main window for some reason.")
			return
		end
		
	    self.wndMain:Show(false, true)
		self.grd_playerlist = self.wndMain:FindChild("grd_playerlist")
		self.btn_ok = self.wndMain:FindChild("OkButton")
		self.windowOpen = false

		if(self.ocr == nil) then
			self.ocr = Apollo.GetAddon("OneClickReport")
		end
		-- if the xmlDoc is no longer needed, you should set it to nil
		-- self.xmlDoc = nil
		
		-- Register handlers for events, slash commands and timer, etc.
		-- e.g. Apollo.RegisterEventHandler("KeyDown", "OnKeyDown", self)
		Apollo.RegisterSlashCommand("lnp", "OnListNearbyPlayersOn", self)


		-- Do additional Addon initialization here
		--Apollo.RegisterEventHandler("SubZoneChanged", "onSubZoneChanged", self)
		self.timer = ApolloTimer.Create(30.000, true, "OnTimer", self)
		Apollo.RegisterEventHandler("UnitCreated", "onUnitCreated", self)
	end
end

-----------------------------------------------------------------------------------------------
-- ListNearbyPlayers Functions
-----------------------------------------------------------------------------------------------
-- Define general functions here

-- on SlashCommand "/lnp"
function ListNearbyPlayers:OnListNearbyPlayersOn()
	self.timer = nil
	self.windowOpen = true
	if(self.ocr == nil) then
		self.btn_ok:Show(false)
	end
	self.wndMain:Invoke() -- show the window
	self.grd_playerlist:DeleteAll()
	for key,val in pairs(nearbyPlayers) do
		if(self.reportedPlayers[key] ~= nil) then
			local timeString = self.reportedPlayers[key]["LastReport"]["nDay"] .. "." .. self.reportedPlayers[key]["LastReport"]["nMonth"] .. "." .. self.reportedPlayers[key]["LastReport"]["nYear"] .. " " .. self.reportedPlayers[key]["LastReport"]["strFormattedTime"]
			self.grd_playerlist:AddRow(key .. " / " .. val['FactionString'] .. " / " .. val['Level'] .. " \nReported " .. self.reportedPlayers[key]["CountReport"] .. " times \nLast Report " .. timeString, "", key)
		else
			self.grd_playerlist:AddRow(key .. " / " .. val['FactionString'] .. " / " .. val['Level'], "", key)
		end
	end
end


-----------------------------------------------------------------------------------------------
-- ListNearbyPlayersForm Functions
-----------------------------------------------------------------------------------------------
-- when the OK button is clicked
function ListNearbyPlayers:OnOK()
	if(self.grd_playerlist:GetCurrentRow() ~= nil) then
		local test = self.grd_playerlist:GetCellData(self.grd_playerlist:GetCurrentRow(), 1)
		if(self.ocr == nil) then
			self.ocr = Apollo.GetAddon("OneClickReport")	
		end
		if(self.ocr ~= nil) then
		
		if(self.reportedPlayers[test] == nil) then
			self.reportedPlayers[test] = {}
			self.reportedPlayers[test]["CountReport"] = 1
		else
			self.reportedPlayers[test]["CountReport"] = self.reportedPlayers[test]["CountReport"] + 1;
		end
		
			self.reportedPlayers[test]["LastReport"] = GameLib.GetLocalTime()
			self.ocr:OnOneClickReportOn("", nearbyPlayers[test]['FactionID'] .. " " .. nearbyPlayers[test]['RaceID'] .. " " .. nearbyPlayers[test]['ClassID'] .. " " .. nearbyPlayers[test]['Name'] .. " " .. nearbyPlayers[test]['Level'] .. " " .. nearbyPlayers[test]['Position'].x .. " " .. nearbyPlayers[test]['Position'].y .. " " .. nearbyPlayers[test]['Position'].z)
		end
	else
		self.ocr:OnOneClickReportOn("", "")
	end
		--self.wndMain:Close() -- hide the window
end

-- when the Cancel button is clicked
function ListNearbyPlayers:OnCancel()
	self.wndMain:Close() -- hide the window
end


function ListNearbyPlayers:onWindowClosed( wndHandler, wndControl )
	self.windowOpen = false
	self.timer = ApolloTimer.Create(30.000, true, "OnTimer", self)
end

function ListNearbyPlayers:onWindowHide( wndHandler, wndControl )
	self.windowOpen = false
	self.timer = ApolloTimer.Create(30.000, true, "OnTimer", self)
end

function ListNearbyPlayers:onGridSelChange()
	
end

function ListNearbyPlayers:OnClear( wndHandler, wndControl, eMouseButton )
	self.grd_playerlist:DeleteAll()
	nearbyPlayers = {}
end

-----------------------------------------------------------------------------------------------
-- ListNearbyPlayers Instance
-----------------------------------------------------------------------------------------------
local ListNearbyPlayersInst = ListNearbyPlayers:new()
ListNearbyPlayersInst:Init()

-----------------------------------------------------------------------------------------------
-- Own Functions
-----------------------------------------------------------------------------------------------

function ListNearbyPlayers:onUnitCreated(unit)
	if(isCity() == false) then
		if(unit:IsACharacter() == true) then 
			local uName = unit:GetName()
			if(nearbyPlayers[uName] == nil) then
				--Print("Added " .. uName)
				local uFactionString = nil
				nearbyPlayers[uName] = {}
				nearbyPlayers[uName]['FactionID'] = unit:GetFaction()
				nearbyPlayers[uName]['RaceID'] = unit:GetRaceId()
				nearbyPlayers[uName]['ClassID'] = unit:GetClassId()
				nearbyPlayers[uName]['Name'] = unit:GetName()
				nearbyPlayers[uName]['Level'] = unit:GetLevel()
				nearbyPlayers[uName]['Position'] = unit:GetPosition()
				nearbyPlayers[uName]['FirstSeen'] = Apollo.GetTickCount()		
				if(nearbyPlayers[uName]['FactionID'] == Unit.CodeEnumFaction.DominionPlayer) then
					uFactionString = "Dominion"
				elseif(nearbyPlayers[uName]['FactionID'] == Unit.CodeEnumFaction.ExilesPlayer) then
					uFactionString = "Exile"
				else
					uFactionString = "Unknown"
				end		
				nearbyPlayers[uName]['FactionString'] = uFactionString
				
				if(self.windowOpen == true) then 
					if(self.reportedPlayers[uName] ~= nil) then
						local timeString = self.reportedPlayers[uName]["LastReport"]["nDay"] .. "." .. self.reportedPlayers[uName]["LastReport"]["nMonth"] .. "." .. self.reportedPlayers[uName]["LastReport"]["nYear"] .. " " .. self.reportedPlayers[uName]["LastReport"]["strFormattedTime"]
						self.grd_playerlist:AddRow(uName .. " / " .. nearbyPlayers[uName]['FactionString'] .. " / " .. nearbyPlayers[uName]['Level'] .. " \nReported " .. self.reportedPlayers[uName]["CountReport"] .. " times \nLast Report " .. timeString, "", uName)
					else
						self.grd_playerlist:AddRow(uName .. " / " .. nearbyPlayers[uName]['FactionString'] .. " / " .. nearbyPlayers[uName]['Level'], "", uName)
					end
				end
			end
		end
	end
end

function ListNearbyPlayers:OnTimer()
	for key,val in pairs(nearbyPlayers) do
		if(Apollo.GetTickCount() - nearbyPlayers[key]['FirstSeen'] > 60000) then
			--Print("Deleted " .. key)
			nearbyPlayers[key] = nil
		end
	end
end

function isCity()
	local currentZone = GameLib.GetCurrentZoneMap()
	
	if(currentZone == nil) then
		return(false)
	end
	
	if(currentZone.strFolder == "Illium" or currentZone.strFolder == "Thayd") then
		return(true)
	else
		return(false)
	end
end


function ListNearbyPlayers:OnSave(eType)
	if eType ~= GameLib.CodeEnumAddonSaveLevel.Realm then return end
	return self.reportedPlayers
end

function ListNearbyPlayers:OnRestore(eType, tSavedData)
	if eType ~= GameLib.CodeEnumAddonSaveLevel.Realm then return end
	self.reportedPlayers = tSavedData
end

